//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/ft/PlainOutput.c,v $
// $Id: PlainOutput.c,v 1.6 2002/11/14 21:58:20 ah Exp $ 
//
// Description: print information about specified wav file
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: PlainOutput.c,v 1.6 2002/11/14 21:58:20 ah Exp $ (c) 2000 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include

#include <AH/Error/Intern.h>
#include <AH/Etech/Types.h>
#include <AH/Etech/Algo.h>
USING_NAMESPACE(AH)
#include "PlainOutput.h"

#include <stdlib.h>
#include <math.h>

// --- local constants ---

// --- local definitions ---

// --- local classes ---

static inline double ld(double val)
{
    return log(val) / log(2);
}


PlainOutput::PlainOutput(FILE* fpo, bool is_linear, unsigned int in_size,
			 unsigned int out_size, double f_max,
			 double db_add, unsigned int debug)
    : fp(fpo), linear(is_linear), inum(in_size), onum(out_size), 
      cnt(0), ocnt(0), debon(debug),
      add_db(db_add), sum_db(0), sum_phi(0), max_db(-1e100), max_phi(-1000),
      min_db(1e100), min_phi(1000), f_min(-1), f_next(0), f1(0), f2(0),
      delta(0)
{
    if (onum < 2) throw ErrorInternBadArg("PlainOutput: onum");
    if (inum < 2) throw ErrorInternBadArg("PlainOutput: inum");

    if (onum < inum)
    {
	f_min = f_max / (inum - 1);
	if (linear)
	{
	    delta = (f_max - f_min) / (onum - 1);
	    f_next = f_min + delta;
	}
	else
	{
	    delta = ld(f_max / f_min) / (onum - 1);
	    f_next = f_min * pow(2.0, delta);
	}
    }
    else 
    {
	onum = inum;
	f_min = f_next = -1;
    }

    if (debon > 1) cerr << "PlainOutput: f_min=" << f_min 
			<< " f_next=" << f_next << endl;
    // print comment line:
    fprintf(fp, "# freq/Hz  dB    degree\n");
}



double PlainOutput::get_dB(const Cplx& val)
{
    double betrag = abs(val);
    if (betrag <= 0.0) return -1000.0;
    return inVdB(betrag);
}



void PlainOutput::print(double f, const Cplx& val)
{
    if (onum == inum) 
    {
	double db = get_dB(val) + add_db;
	double phi = inDeg(arg(val));
	printValues(f, db, phi, db, phi, db, phi);
	++ocnt;
	return;
    }
    if (f <= 0) 
    {
	// ignore DC point
	double db = get_dB(val) + add_db;
	double phi = inDeg(arg(val));
	printValues(f, db, phi, db, phi, db, phi);
	++ocnt;
	if (linear) f_next = f_min + (ocnt * delta);
	else        f_next = f_min * pow(2.0, ocnt * delta);
	cnt = 0;
	return;
    }
    if (f > f_next)
    {
	// flush buffered data:
	double db  = sum_db/cnt;
	double phi = sum_phi/cnt;
	printValues(sqrt(f1 * f2), db, phi, max_db, max_phi, min_db, min_phi);
	++ocnt;
	if (linear) f_next = f_min + (ocnt * delta);
	else        f_next = f_min * pow(2.0, ocnt * delta);
	cnt = 0;
	sum_db = sum_phi = 0;
    }
    if (0 == cnt)
    {
	max_db = min_db = sum_db = get_dB(val) + add_db;
	max_phi = min_phi = sum_phi = inDeg(arg(val));
	f1 = f2 = f;
    }
    else
    {
	double db = get_dB(val) + add_db;
	double phi = inDeg(arg(val));
	sum_db += db;
	sum_phi += phi;
	if (db > max_db) max_db = db;
	if (db < min_db) min_db = db;
	if (phi > max_phi) max_phi = phi;
	if (phi < min_phi) min_phi = phi;
	f2 = f;
    }
    ++cnt;
}


void PlainOutput::printValues(double f, double db, double phi, double dbmax,
			      double phimax, double dbmin, double phimin)
{
    fprintf(fp, "%f %f %f %f %f %f %f\n", 
	    f, db, phi, dbmax, phimax, dbmin, phimin);
}


void PlainOutput::done()
{
    if (cnt)
    {
	double db     = sum_db/cnt;
	double phi    = sum_phi/cnt;
	printValues(sqrt(f1 * f2), db, phi, max_db, max_phi, min_db, min_phi);
    }
}



// --- end of file ---




