//---------------------------------------------------------------------------
// Copyright 2002 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/ft/testPlainOutput.c,v $
// $Id: testPlainOutput.c,v 1.1 2002/11/02 00:35:36 ah Exp $ 
//
// Description: 
//
//---------------------------------------------------------------------------

const char main_CVSID[] = "$Id: testPlainOutput.c,v 1.1 2002/11/02 00:35:36 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include <stdlib.h>
#ifdef NO_UNISTD
#   include <getopt.h>
#else
#   include <unistd.h>
#endif
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <iostream>
#include <string>
USING_NAMESPACE(std)

#include <AH/Etech/Algo.h>
#include <AH/Etech/Riaa.h>
#include <AH/Error/Class.h>
#include <AH/Error/Sys.h>
USING_NAMESPACE(AH)

#include "PlainOutput.h"

// --- locaL definitions ---

#define USE_TEST_SIG 1

// --- local constants ---

static const char* cmdLineOptions = "d";

// --- local data ---

static unsigned int debon = 0;

// --- local classes ---

// --- local functions ---


static unsigned int process(unsigned int inum,
			    unsigned int onum)
{
    // create complex spectrum data array
    Cplx* spectrum = new Cplx[inum];
    if (!spectrum)
	throw ErrorSysOutOfMem();

    // fill spectrum data:
    const double f_max = 40000;
    const double f_inc = f_max / (double)(inum-1);
    double f = 0;
    for (unsigned int i = 0; i < inum; ++i)
    {
	spectrum[i] = RiaaPreEmphasis(f);
	f += f_inc;
    }

    //---------------------------------------------------
    // write the unreduced results into the output file:
    //---------------------------------------------------

    // open output file
    FILE* fp = fopen("testplain_raw.txt", "w");
    if (!fp) 
    {
	cerr << "Cannot open testplain_raw.txt: " << strerror(errno) << endl;
	return 1;
    }

    // print entries:
//    PlainOutput raw(fp, inum, inum, f_max, debon);
    fprintf(fp, "# freq/Hz  dB    degree\n");
    f = 0;
    for (unsigned int i = 0; i < inum; ++i)
    {
	double db = inVdB(abs(spectrum[i]));
	double phi = inDeg(arg(spectrum[i]));
	fprintf(fp, "%f %f %f\n", f, db, phi);
//	printValues(f, db, phi);
//	raw.print(f, spectrum[i]);
	f += f_inc;
    }
//    raw.done();
    fclose(fp);

    //---------------------------------------------------
    // write the unreduced results into the output file:
    //---------------------------------------------------

    // open output file
    fp = fopen("testplain_red.txt", "w");
    if (!fp) 
    {
	cerr << "Cannot open testplain_red.txt: " << strerror(errno) << endl;
	return 1;
    }

    // print entries:
    PlainOutput reduced(fp, inum, onum, f_max, debon);
    f = 0;
    for (unsigned int i = 0; i < inum; ++i)
    {
	reduced.print(f, spectrum[i]);
	f += f_inc;
    }
    reduced.done();
    fclose(fp);

    return 0;
}


// --- global functions ---


int main(int argc, char * const argv[])
{
    // loop through command line options:
    int option;
    while ((option = getopt(argc, argv, cmdLineOptions)) != -1)
    {
	switch (option) 
	{
	case 'd': ++debon; break;
	}
    }

    static unsigned int errnum;
    try { errnum = process(10000, 100); }
    catch (exception& err)
    {
	++errnum;
	cerr << "Error: " << err.what() << endl;
    }
    
    exit (errnum ? 1 : 0);
    return  (errnum ? 1 : 0); // to keep MSVC quiet
} // end of main()


// --- end of file ---
