const char sweep_CVSID[] = "$Id: wisdom.c,v 1.2 2004/04/21 20:40:35 ah Exp $ (c) 2002 Andreas Huennebeck";

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
USING_NAMESPACE(AH)

#include "wisdom.h"
#include <drfftw.h> 
#include <string.h> 
#include <errno.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <string> 
#include <iostream> 

#include <AH/Error/Basic.h>

// --- local constants ---

// --- local data ---

// --- local classes ---

// --- local functions ---

// --- global functions ---

FtWisdom::FtWisdom(unsigned int debug)
    : fileName("/usr/share/fftw/wisdom_r"), plan(0), size(0), debon(debug)
{
    // import wisdom
    this->importFromFile();
}


FtWisdom::~FtWisdom()
{
    // destroy existing plan
    if (plan) fftw_destroy_plan(plan);

    // export wisdom
    this->exportToFile();
}


void FtWisdom::importFromFile()
{
    // import wisdom
    FILE* fp = fopen(fileName.c_str(), "r");
    if (fp)
    {
	if (debon > 2)
	    std::cerr << "FtWisdom::importFromFile: import wisdom" << std::endl;
	fftw_import_wisdom_from_file(fp);
	fclose(fp);
    }
    else std::cerr << "Error reading " << fileName << ": "
	      << strerror(errno) << std::endl;
}


void FtWisdom::exportToFile()
{
    // make wisdom accessible for all users:
    mode_t curmask = umask(0);

    // export wisdom
    FILE* fp = fopen(fileName.c_str(), "w");
    if (fp)
    {
	if (debon > 2)
	    std::cerr << "FtWisdom::exportToFile: export wisdom" << std::endl;
	fftw_export_wisdom_to_file(fp);
	fclose(fp);
    }
    else std::cerr << "Error creating " << fileName << ": "
	      << strerror(errno) << std::endl;

    umask(curmask);
}



rfftw_plan FtWisdom::getPlan(unsigned int num)
{
    if (plan)
    {
	if (num == size) return plan;
	rfftw_destroy_plan(plan);
    }

     /* create a plan for N=num, possibly creating and/or using wisdom */
    if (debon > 1) 
	std::cerr << "FtWisdom::getPlan: create plan for N=" << num << std::endl;
    plan = rfftw_create_plan(num, FFTW_FORWARD,
			    FFTW_MEASURE | FFTW_USE_WISDOM);
    if (0 == plan) throw ErrorBasic("Cannot create FFTW plan");
    if (debon > 2)
	std::cerr << "FtWisdom::getPlan: plan for N=" << num << " created" << std::endl;
//    this->exportToFile(); // just in case
    size = num;
    return plan;
}


// --- end of file ---
