//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/Algo.h,v $
// $Id: Algo.h,v 1.6 2002/10/26 22:52:30 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_ALGO_H	// prevent multiple includes
#define ETECH_ALGO_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Voltage.h>
#include <AH/Etech/Current.h>
#include <AH/Etech/Power.h>
#include <AH/Etech/Impedance.h>

// --- name space ---

BEGIN_NAMESPACE(Ah)

// --- types and classes ---

// --- function prototypes ---

/// overloaded operator / for division of {\tt Voltage} / {\tt Current}
inline Impedance operator / (const Voltage &u, const Current &i)
	{ return Impedance(u.getVal() / i.getVal()); }

/// overloaded operator * for multiplication of {\tt Impedance} * {\tt Current}
inline Voltage operator * (const Impedance &r, const Current &i)
	{ return Voltage(r.getVal() * i.getVal()); }
/// overloaded operator * for multiplication of {\tt Current} * {\tt Impedance}
inline Voltage operator * (const Current &i, const Impedance &r)
	{ return Voltage(i.getVal() * r.getVal()); }
/// overloaded operator / for division of {\tt Power} / {\tt Current}
inline Voltage operator / (const Power&p, const Current &i)
	{ return Voltage(p.getVal() / i.getVal()); }

/// overloaded operator / for division of {\tt Voltage} / {\tt Impedance}
inline Current operator / (const Voltage &u, const Impedance &r)
	{ return Current(u.getVal() / r.getVal()); }
/// overloaded operator / for division of {\tt Power} / {\tt Voltage}
inline Current operator * (const Power&p, const Voltage &u)
	{ return Current(p.getVal() / u.getVal()); }

/// overloaded operator * for multiplication of {\tt Voltage} * {\tt Current}
inline Power operator * (const Voltage &u, const Current &i)
	{ return Power(u.getVal() * i.getVal()); }
/// overloaded operator * for multiplication of {\tt Current} * {\tt Voltage}
inline Power operator * (const Current &i, const Voltage &u)
	{ return Power(i.getVal() * u.getVal()); }

/// return value in voltage decibel
inline Dbl inVdB (Dbl x) { return (20 * log10(x)); }
/// return value in power decibel
inline Dbl inPdB (Dbl x) { return (10 * log10(x)); }
/// return phase argument in degree
inline Dbl inDeg (Dbl x) { return (x * 180.0/acos(0.0)); }

// --- name space ---

END_NAMESPACE

#endif // ETECH_ALGO_H

