//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/FET.h,v $
// $Id: FET.h,v 1.5 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_FET_H	// prevent multiple includes
#define ETECH_FET_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Transistor.h>
#include <AH/Etech/Voltage.h>

USING_NAMESPACE(Ah)

// --- types and classes ---

BEGIN_NAMESPACE(Ah)

/// represents a FET
/**
 {\tt FET} represents a field effect transistor. It can be used both 
 in DC (direct current) and in AC (alternate current) applications.
 */
class FET: public Transistor
{
  protected:
    static const Voltage Up25;
    Current Ids;

  public:
    /// default constructor
    FET(const Current& i_ds) : Ids(i_ds) {}
    /// copy constructor
    FET(const FET& t) : Ids(t.Ids) {}

    /// destructor:
    virtual ~FET() {}

    /// get steilheit
    virtual Cplx getS(const Voltage& Ugs, Dbl t = PhysConst_T25) const;
//    virtual Cplx getS(const Voltage& Ugs, Dbl t = PhysConst::t25) const;
    /// get steilheit
    virtual Cplx getS(const Current& Id, Dbl t = PhysConst_T25) const;
//    virtual Cplx getS(const Current& Id, Dbl t = PhysConst::t25) const;

    /// return output current depending on input voltage
    virtual Current getIout(const Voltage& Uin, 
			    Dbl t = PhysConst_T25) const;
//			    Dbl t = PhysConst::t25) const;

    /// get Up at specified temperature
    static Voltage getUp(Dbl t = PhysConst_T25);
//    static Voltage getUp(Dbl t = PhysConst::t25);

}; // class FET


END_NAMESPACE

#endif // ETECH_FET_H
