//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/Etech/OpAmp.h,v $
// $Id: OpAmp.h,v 1.13 2001/10/05 23:44:27 ah Exp $
//
//---------------------------------------------------------------------------

#ifndef ETECH_OPAMP_H	// prevent multiple includes
#define ETECH_OPAMP_H

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Impedance.h>

USING_NAMESPACE(Ah)
BEGIN_NAMESPACE(Ah)

// --- forward declarations ---

class Resistor;
class Capacitor;

// --- types and classes ---


/// represents an operational amplifier
/**
 {\tt OpAmp} represents a real operational amplifier which can be used both 
 in DC (direct current) and in AC (alternate current) applications.
 */
class OpAmp
{
  private:
    Impedance	Zin;	// calculated from Rin and Cin(f)

  protected:
    Dbl		f;	// current frequency (just for caching)
    Cplx	Af;	// open loop amplification at f (just for caching)

    const char*	Name;	// name of Operational amplifier
    Dbl		A0;	// open loop DC amplification
    Dbl		Fg;	// open loop crossover frequency

    Dbl		En;	// high frequency noise voltage density
    Dbl		FgEn;	// 1/f voltage noise crossover frequency

    Dbl		In;	// low frequency current noise density
    Dbl		FgIn;	// current noise crossover frequency

    Resistor*	Rout;	// open loop output impedance
    Resistor*   Rin;	// input impedance
    Capacitor*	Cin;    // input capacitance

  public:
    /// default constructor
    OpAmp();
    /// simple constructor
    OpAmp(const char* name, Dbl a0, Dbl ft);
    /// complete constructor
    OpAmp(const char* name, Dbl a0, Dbl ft, Dbl rout, Dbl en=0, 
	  Dbl fgen=0.001, Dbl in=0, Dbl fgin=0.001);
    /// copy constructor
    OpAmp(const OpAmp& op);
    /// destructor:
    virtual ~OpAmp();

    /// return name of operational amplifier
    const char* getName() const { return Name; }

    /// get differential open loop amplification at frequency {\tt freq}
    virtual const Cplx* getAd(Dbl freq);

    /// get open loop input impedance at frequency {\tt freq}
    virtual const Impedance* getZin(Dbl freq);

    /// get open loop output impedance at frequency {\tt freq}
    virtual const Impedance* getZout(Dbl freq);

    /// assign operator
    const OpAmp& operator = (const OpAmp& op);

}; // class OpAmp


END_NAMESPACE

#endif // ETECH_OPAMP_H
