//---------------------------------------------------------------------------
// Copyright 2000 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Mpeg/FrameHeader.h,v $
// $Id: FrameHeader.h,v 1.3 2000/12/04 19:31:46 ah Exp $ 
//
// Description: class representing a MPEG Header
//
//---------------------------------------------------------------------------

#ifndef AH_IO_MPEG_FRAMEHEADER_H
#define AH_IO_MPEG_FRAMEHEADER_H

// --- necessary includes ---
#include <AH/Config/Platform.h> // always the first include
#include <AH/Config/Types.h>
USING_NAMESPACE(AH)

#include <string>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/// class representing a MPEG Header
class IOMpegFrameHeader
{
 public:
    /// maximum size of buffer for frame
#ifdef NO_ANSI_CPLUSPLUS
    enum {MAX_FRAME_SIZE = (144 * 448) / 8 + 1};
#else
    static const int MAX_FRAME_SIZE = (144 * 448) / 8 + 1;
#endif
    /// constructor
    IOMpegFrameHeader(const uint8* frameData);
    /// destructor
    ~IOMpegFrameHeader() {}

    // access functions:

    /// return true if frame is valid
    bool validHeader() const;

    /// get version of MPEG: 1=MPEG1, 2=MPEG2, 3=MPEG2.5, -1=invalid
    int getMpegVersion() const;
    /// get layer: 1/2/3=Layer 1/2/3, -1=invalid
    int getLayer() const;
    /// get bit rate in kBit/sec: 0=variable, -1=invalid
    int getBitRate() const;
    /// get sample rate in kSamples/sec: -1=invalid
    int getSampleRate() const;
    /// get padding byte
    int getPadding() const;
    /// return true if frame contains CRC checksum
    bool hasCRC() const;

    /// get frame size
    int getFrameSize() const;
    /// get CRC data size
    int getCrcSize() const;
    /// get audio data size
    int getAudioSize() const;
    /// get audio time in ms
    double getFrameTime() const;

    /// get info string
    string getInfo() const;

    /// return true if byte is first byte of a MPEG frame header
    static bool isFirstByte(uint8 byte);
    /// return true if byte is second byte of a MPEG frame header
    static bool isSecondByte(uint8 byte);

 private:
    uint8 bytes[4]; // local copy of frame header
};

END_NAMESPACE

#endif // AH_IO_MPEG_FRAMEHEADER_H
// --- end of file ---
