//---------------------------------------------------------------------------
// Copyright 1999 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file 'COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Riff/WaveFmtChunk.h,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

#ifndef AH_IO_RIFF_WAVE_FMT_CHUNK_H		// prevent multiple includes
#define AH_IO_RIFF_WAVE_FMT_CHUNK_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Config/Types.h>
#include <AH/IO/Riff/Chunk.h>
USING_NAMESPACE(AH)

#include <iostream>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)

class IORiffWaveFmtChunk: public IORiffChunk
{
    class Specific
    {
     public:
	Specific(uint16 k) : key(k) {}
	virtual ~Specific() {}
	virtual ostream& printFormat(ostream& out, const char* pre = 0);
	virtual ostream& printInfo(ostream& out, const char* pre = 0)
	    { return out; }
     private:
	uint16 key;
    };

    class MsPcm: public Specific
    {
     public:
	MsPcm(const unsigned char* data) : 
	    Specific(0x0001), sampleSize((const uint16*)data) {}
	virtual ~MsPcm() {}
	uint32 getSampleSize() const { return *sampleSize; }
	virtual ostream& printFormat(ostream& out, const char* pre = 0);
	virtual ostream& printInfo(ostream& out, const char* pre = 0);
     private:
	const uint16* sampleSize;
    };

 public:
    /// constructor
    IORiffWaveFmtChunk(const unsigned char* data);
    /// destructor
    virtual ~IORiffWaveFmtChunk() { delete specData; }

    /// get pointer to next chunk:
    virtual const unsigned char* nextChunk()
	{ return (IORiffChunk::nextChunk() + 4); }

    /// get pointer to chunk data:
    virtual const unsigned char* chunkData()
	{ return (IORiffChunk::chunkData() + 14); }

    // access functions:
    /// return format key
    uint32 getFormatKey() const { return format; }
    /// return number of channels
    uint32 getChannels() const { return channels; }
    /// return sample rate in samples per second
    uint32 getSampleRate() const { return sampleRate; }
    /// return data rate in bytes per second
    uint32 getDataRate() const { return dataRate; }
    /// return block size
    uint32 getBlockSize() const { return blockSize; }

    /// print information about this chunk
    virtual ostream& printInfo(ostream& out, const char* pre = 0);
 private:
    uint16	format;
    uint16	channels;
    uint32	sampleRate;	// in samples per second
    uint32	dataRate;	// in bytes per second
    uint16	blockSize;
    Specific*	specData;	// specific data
};

END_NAMESPACE

#endif // !AH_IO_RIFF_WAVE_FMT_CHUNK_H
