//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/include/AH/IO/Smart/FilePtr.h,v $
// $Id: FilePtr.h,v 1.2 2004/04/20 22:25:30 ah Exp $
//
// Description: 
//
//---------------------------------------------------------------------------

#ifndef AH_IO_SMART_FILEPTR_H		// prevent multiple includes
#define AH_IO_SMART_FILEPTR_H

// --- forward declarations ---

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Error/Sys.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <string>
USING_NAMESPACE(std)

// --- types and classes ---

BEGIN_NAMESPACE(AH)

/// smart <tt>stdio</tt> file pointer
/**
   Use <tt>AH::IOSmartFilePtr</tt> if you need a <tt>FILE</tt> pointer 
   to a file which is closed properly if the scope is left, e.g.
   due to an exception.

   Please include <em>AH/IO/Smart/FilePtr.h</em>.
 */
class IOSmartFilePtr
{
 public:
    /// constructor for file pointer
    IOSmartFilePtr
    (
	/// name of file
	const string& name, 
	/// file mode: "r" for read, "w" for write
	const string& mode
    )
	: fp(fopen(name.c_str(), mode.c_str()))
    {
	if (!fp) throw ErrorSys("cannot open " + name);
    }

    /// destructor
    ~IOSmartFilePtr() { close(); }

    /// close file
    void close() 
    {
	if (fp)
	{
	    (void)fclose(fp);
	    fp = 0;
	}
    }

    /// return pointer to file
    FILE* operator()() { return fp; }

 private:
    FILE*	fp;	// the file pointer itsself

};	// IOSmartFileptr


END_NAMESPACE
#endif // !AH_IO_SMART_FILEPTR_H




