//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// $Source: /home/ah/CvsTree/daten/src/libAHetech/Filter2PoleInv.c,v $
// $Id: Filter2PoleInv.c,v 1.2 2001/10/05 23:44:26 ah Exp $
//
// Description: implementation of class Filter2PoleInv
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: Filter2PoleInv.c,v 1.2 2001/10/05 23:44:26 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/Config/Platform.h>
#include <AH/Etech/Filter2PoleInv.h>
#include <AH/Etech/MathConst.h>
#include <AH/Etech/Resistor.h>
#include <AH/Etech/Capacitor.h>

// --- member functions ---

// constructor
Filter2PoleInv::Filter2PoleInv
(
    const OpAmp& opamp, 
    const Impedance& z1, 
    const Impedance& z2, 
    const Impedance& z3, 
    const Impedance& z4, 
    const Impedance& z5
)
    : op(opamp), Z1(z1), Z2(z2), Z3(z3), Z4(z4), Z5(z5) //, filter(Unknown)
{
}

#if 0
// constructor
Filter2PoleInv(const OpAmp& opamp, type t, Dbl fg, Dbl a1, Dbl a2)
    : op(opamp), Z1(0), Z2(0), Z3(0), Z4(0), Z5(0), filter(t)
{
}
#endif

// get amplification at frequency 'freq':
Cplx Filter2PoleInv::getA(Dbl freq)
{
    //-----------------------------------------------------------------------
    //                                     1
    // A = - ----------------------------------------------------------------
    //       Z1/Z2 - Z1/(Z4*Ad) + (1/Ad+(1+1/Ad)*Z4/Z5)*(1+Z1/Z2+Z1/Z3+Z1/Z4)
    //-----------------------------------------------------------------------

    const Cplx* Ad = op.getAd(freq);

    Cplx k1 = Z1/Z2;

    Cplx k2 = Z4/Z5;
    if (Ad) k2 = 1/(*Ad) + (1 + 1/(*Ad)) * k2;

    Cplx k3 = 1 + k1 + Z1/Z3 + Z1/Z4;

    Cplx nenner = k1;
    if (Ad) nenner = nenner - (Z1/Z4)/(*Ad);
    nenner = nenner + k2 * k3;

    Cplx A = -1 / nenner;
    return (A);
}

bool Filter2PoleInv::calcLowpass
(
    Dbl fg, 
    Dbl A0, 
    Dbl a1, 
    Dbl b1, 
    const Capacitor& C1, 
    const Capacitor& C2, 
    Resistor& R1,
    Resistor& R2,
    Resistor& R3
)
{
    double k1 = a1 * C2.getCap();
    double k2 = k1 * k1 - 4 * C1.getCap() * C2.getCap() * b1 * (1+A0);
    if (k2 < 0) return false;	// wegen der Wurzel

    double k3 = 4 * MathConst::Pi * fg * C1.getCap() * C2.getCap();
    R2 = (k1 - sqrt(k2)) / k3;
    R1 = R2.getRes() / A0;
    R3 = b1 / (k3 * MathConst::Pi * fg * R2.getRes());

    return true;
}
