//---------------------------------------------------------------------------
// Copyright 1998 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHio/TestCFormat.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char rcsid[] = "$Id: TestCFormat.c,v 1.6 2005/09/21 23:54:48 ah Exp $ (c) 1998 Andreas Huennebeck";

// --- necessary includes ---

#include <AH/IO/CFormat.h>
USING_NAMESPACE(AH)

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <sstream>

USING_NAMESPACE(std)

#include <AH/Test/Log.h>

// --- local implementation ---

static bool debug = false;

//---------------------------------------------------------------
// test function for testing one argument using both IOUtilCFormat
// and sprintf and comparing the results
//---------------------------------------------------------------
template <class Arg> static void test_format
(
    TestLog& testlog,
    const char* format,		// format string
    const Arg& arg		// argument
)
{
    ostringstream what;
    what << "IOCFormat(\"" << format << "\") << \"" << arg << "\""; 
    testlog.announceTest(what.str());

    //------------------------------------------
    // apply IOUtilCFormat, store result in buf1:
    //------------------------------------------
    ostringstream buf1;
    buf1 << IOCFormat(format) << arg << '\0';
    buf1.flush();

    testlog.returned(buf1.str());

    //--------------------------------
    // apply sprintf, result in buf2:
    //--------------------------------
    char buf2[128];
    (void)sprintf(buf2,format,arg);

    //------------------
    // compare results:
    //------------------
    if (0 != strcmp(buf1.str().c_str(), buf2))
    {
	//--------------------------------------------------------------
	// try to convert the output back into doubles and compare them
	//--------------------------------------------------------------
	bool comparison = true;
	char* endp;
	double val1 = strtod(buf1.str().c_str(), &endp);
	if (endp == buf1.str().c_str())	
	    comparison = false;		// no conversion to double done
	else
	{
	    double val2 = strtod(buf2, &endp);
	    if ((endp == buf2) || (val1 != val2))
		comparison = false;	// no conversion or unequal 
	}
	if (comparison)
	    testlog.passed();
	else
	    testlog.failed(buf2);
    }
    else testlog.passed();
}

//----------------------------------------------------------------------
// test data for integer arguments:
//----------------------------------------------------------------------

static int intArgs[] = 
{
    1, 255, 2345
};

static const char* intFormats[] = 
{
    // decimal output:
    "%2d", "%4d", "%10d", "%010d", "%-10d",
    // hexadecimal output:
    "%x", "%X", "%2x", "%2X", "%8x", "%8X", "%#x", "%#X", "%#4x", "%#4X",
    "%#10x", "%#10X",
    // octal output:
    "%o", "%3o", "%12o", "%#o", "%#5o", "%#14o"
};

//----------------------------------------------------------------------
// test data for double arguments:
//----------------------------------------------------------------------

static double doubleArgs[] = 
{
    2.1, 0.010, 84.22152,0.0000213, 64.83394, 12.01882, 46.21371
};

static const char* doubleFormats[] = 
{
    "%5.2f", "%10.3f", "%8.3f", "%11.5f", "%5.0f", 
    "%e", "%E", "%5.2e", "%5.2E"

};

//----------------------------------------------------------------------
// test data for string arguments:
//----------------------------------------------------------------------

static const char* stringArgs[] = 
{
    "hello", "world", "hello world"
};

static const char* stringFormats[] = 
{
    "%s", "%2s", "%10s", "%-10s", "%20s"
};


//----------------------------------------------------------------------
// --- global implementation ---
//----------------------------------------------------------------------


int main(int argc, char* argv[])
{
    if ((2 <= argc) && (0 == strcmp("-d", argv[1])))
	debug = true;

    TestLog testlog(cout, "class IOUtilCFormat", debug);

    //-------------------------
    // test integer arguments:
    //-------------------------
    testlog.announceSeries("integer arguments");
    int f, a;
    int fsize = sizeof(intFormats)/sizeof(intFormats[0]);
    int asize = sizeof(intArgs)/sizeof(intArgs[0]);
    for (f=0; f < fsize; f++)
    {
	for (a=0; a < asize; a++)
	{
	    test_format(testlog, intFormats[f], intArgs[a]);
	    test_format(testlog, intFormats[f],-intArgs[a]);
	}
    }

    //--------------------------
    // test floating arguments:
    //--------------------------
    testlog.announceSeries("floating point arguments");
    fsize = sizeof(doubleFormats)/sizeof(doubleFormats[0]);
    asize = sizeof(doubleArgs)/sizeof(doubleArgs[0]);
    for (f=0; f < fsize; f++)
    {
	for (a=0; a < asize; a++)
	{
	    test_format(testlog, doubleFormats[f], doubleArgs[a]);
	    test_format(testlog, doubleFormats[f],-doubleArgs[a]);
	}
    }

    //------------------------
    // test string arguments:
    //------------------------
    testlog.announceSeries("string arguments");
    fsize = sizeof(stringFormats)/sizeof(stringFormats[0]);
    asize = sizeof(stringArgs)/sizeof(stringArgs[0]);
    for (f=0; f < fsize; f++)
    {
	for (a=0; a < asize; a++)
	{
	    test_format(testlog, stringFormats[f], stringArgs[a]);
	}
    }

    //----------------
    // print results:
    //----------------
    return testlog.makeReport();
}
