//---------------------------------------------------------------------------
// Copyright 1997 Andreas Huennebeck
//
// Licensed under the terms of the GNU General Public License.
// ABSOLUTELY NO WARRANTY. See the file `COPYING' in this distribution.
//
// $Source: /home/ah/CvsTree/daten/src/libAHstl/StringPrintf.c,v $
// $Id: 
//
// Description: 
//
//---------------------------------------------------------------------------

static const char cvsid[] = "$Id: StringPrintf.c,v 1.1 2004/04/04 22:41:52 ah Exp $ (c) 1997 Andreas Huennebeck";

// --- necessary includes ---

#include <stdio.h>
#include <stdarg.h>
#include <AH/STL/String.h>
USING_NAMESPACE(AH)

BEGIN_NAMESPACE(AH)


static const unsigned int LOCAL_BUFSIZE = 1024;

// printf compatible format string into STL string
string STLStringPrintf (unsigned int maxSize, const char* format, ...)
{
    // check for existing message:
    if ((NULL != format) && ('\0' != *format))
    {
	va_list args;
	va_start(args, format);

	// let's prevent heap allocation for short messages:
	// - use a local buffer, and allocate from heap only for 
	//   messages which does not fit into the local buffer
	char local_buffer[LOCAL_BUFSIZE + 1];
	char* msg = local_buffer;
	if (maxSize > LOCAL_BUFSIZE)
	{
	    msg = new char(maxSize+1);	// buffer on heap
	    if (msg)
	    {
		msg = local_buffer;
		maxSize = LOCAL_BUFSIZE;
	    }
	}

	int ret = vsnprintf(msg, maxSize, format, args);
	va_end(args);
	if (0 < ret)
	{
	    msg[maxSize-1] = '\0'; // terminate string, maxSize-1 is correct
	    string message(msg);
	    if (msg != local_buffer)
		delete[] msg;		// buffer on heap
	    return message;
	}
	if (msg != local_buffer)
	    delete[] msg;		// buffer on heap
    }
    return "";
}


END_NAMESPACE
